#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_statistics.h>
#include <gsl/gsl_errno.h>

#include "vast_limits.h"
#include "stetson_variability_indexes.h" // for MAD computation
#include "detailed_error_messages.h" // for report_lightcurve_statistics_computation_problem()
#include "index_vs_mag.h"


int main(){
 
 double tmp_d_value_to_print;
 int output_file_counter;
 FILE * output_filedescriptor;
 
 double **index; // the actual values of all indexes for all stars
 double **index_expected; // the expected (for non-variables) values of all indexes for all stars
 double **index_spread; // the RMS (?) scatter of the the expected (for non-variables) values of all indexes for all stars
 double **index_filtered;
 double **index_spread_filtered;
 char **lightcurvefilename;
 size_t * sort_index;
 double * data_for_stat;
 double * data_for_stat_spread;
 int * index_for_stat;
 double * array_of_magnitudes;
 char string_to_parse[MAX_STRING_LENGTH_IN_VAST_LIGHTCURVE_STATISTICS_LOG];
 char substring_to_parse[MAX_STRING_LENGTH_IN_VAST_LIGHTCURVE_STATISTICS_LOG];
 
 double tmpmag,tmpsigma,tmpdouble;
 
 int n_stars_in_lightcurve_statistics_file; // number of stars in the input statistics file
 
 int i,j,k,l; // counters
 
 int start_index,stop_index;
 int varindex_counter;
 
 int star_half_mag_away; // used to make sure our bins are not narrower than 0.5 mag

 int bright_star_half_mag_away; 

 FILE * lightcurve_statistics_file;
 
 FILE * lightcurve_statistics_expected_file;
 FILE * lightcurve_statistics_spread_file;
 FILE * lightcurve_statistics_expected_plus_spread_file;
 FILE * lightcurve_statistics_expected_minus_spread_file;

 FILE * lightcurve_statistics_normalized_file;
 
 // To compute the measures of variability selection efficiency C P F
 FILE * list_of_known_vars_file; 
 char ** known_variable_lightcurvefilename;
 int n_known_variables;
 double threshold; // select as variable stars the ones that stand out by more than threshold*sigma in some index
 int number_of_selected_objects[MAX_NUMBER_OF_INDEXES_TO_STORE];
 int number_of_selected_variables[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double C; // Number of selected variables/Total number of confirmed variables
 double P; // Number of selected variables/Total number of selected objects
 double F; // 2 * (C * P)/(C + P)
 char short_index_name[256];
 // to compute maximum efficiency
 FILE * vast_detection_efficiency_log;
 // Naturally, there will be different values of Fmax, C_at_Fmax, and P_at_Fmax for each index
 double Fmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double C_at_Fmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double P_at_Fmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double threshold_at_Fmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double threshold_at_Cmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double P_at_Cmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double Cmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double fraction_of_obj_rejected_at_Cmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 double fraction_of_obj_rejected_at_Fmax[MAX_NUMBER_OF_INDEXES_TO_STORE];
 FILE * lightcurve_statistics_expected_plus_spread_file_Cmax;
 char comments_string_autocandidates[2048];
 //
 char full_string_autocandidates[MAX_STRING_LENGTH_IN_LIGHTCURVESTATS_FILE];
 
 // Initialize Fmax, j just because I use this counter with MAX_NUMBER_OF_INDEXES_TO_STORE later
 for(j=0;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
  Fmax[j]=0.0;
 }
 
 // Read the indexes
 lightcurve_statistics_file=fopen("vast_lightcurve_statistics.log","r");
 if( lightcurve_statistics_file==NULL ){
  fprintf(stderr,"ERROR: Can't open file vast_lightcurve_statistics.log !\n");
  report_lightcurve_statistics_computation_problem();
  return 1;
 }
 // count lines in the file
 n_stars_in_lightcurve_statistics_file=0;
 while( NULL!=fgets(string_to_parse,MAX_STRING_LENGTH_IN_LIGHTCURVESTATS_FILE,lightcurve_statistics_file) )n_stars_in_lightcurve_statistics_file++;
 fseek(lightcurve_statistics_file,0,SEEK_SET); // go back to the beginning of the file
 // allocate memory
 index=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double *));
 if( index==NULL ){fprintf(stderr,"Memory allocation ERROR: index\n");return 1;}
 index_expected=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double *));
 if( index_expected==NULL ){fprintf(stderr,"Memory allocation ERROR: index_expected\n");return 1;}
 index_filtered=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double *));
 if( index_filtered==NULL ){fprintf(stderr,"Memory allocation ERROR: index_filtered\n");return 1;}
 index_spread=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double *));
 if( index_spread==NULL ){fprintf(stderr,"Memory allocation ERROR: index_spread\n");return 1;}
 index_spread_filtered=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double *));
 if( index_spread_filtered==NULL ){fprintf(stderr,"Memory allocation ERROR: index_spread_filtered\n");return 1;}
 lightcurvefilename=malloc(n_stars_in_lightcurve_statistics_file*sizeof(char *));
 if( lightcurvefilename==NULL ){fprintf(stderr,"Memory allocation ERROR: lightcurvefilename\n");return 1;}
 known_variable_lightcurvefilename=malloc(n_stars_in_lightcurve_statistics_file*sizeof(char *));
 if( known_variable_lightcurvefilename==NULL ){fprintf(stderr,"Memory allocation ERROR: known_variable_lightcurvefilename\n");return 1;}
 sort_index=malloc(n_stars_in_lightcurve_statistics_file*sizeof(size_t));
 if( sort_index==NULL ){fprintf(stderr,"Memory allocation ERROR: sort_index\n");return 1;}
// data_for_stat=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double));
// if( data_for_stat==NULL ){fprintf(stderr,"Memory allocation ERROR: data_for_stat\n");return 1;}
// data_for_stat_spread=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double)); 
// if( data_for_stat_spread==NULL ){fprintf(stderr,"Memory allocation ERROR: data_for_stat_spread\n");return 1;}
 array_of_magnitudes=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double));
 if( array_of_magnitudes==NULL ){fprintf(stderr,"Memory allocation ERROR: array_of_magnitudes\n");return 1;}
 index_for_stat=malloc(n_stars_in_lightcurve_statistics_file*sizeof(int));
 if( index_for_stat==NULL ){fprintf(stderr,"Memory allocation ERROR: index_for_stat\n");return 1;}
 for(i=0;i<n_stars_in_lightcurve_statistics_file;i++){
  index[i]=malloc(MAX_NUMBER_OF_INDEXES_TO_STORE*sizeof(double));if( index[i]==NULL ){fprintf(stderr,"Memory allocation ERROR: index[%d]\n",i);return 1;}
  index_expected[i]=malloc(MAX_NUMBER_OF_INDEXES_TO_STORE*sizeof(double));if( index_expected[i]==NULL ){fprintf(stderr,"Memory allocation ERROR: index_expected[%d]\n",i);return 1;}
  index_filtered[i]=malloc(MAX_NUMBER_OF_INDEXES_TO_STORE*sizeof(double));if( index_filtered[i]==NULL ){fprintf(stderr,"Memory allocation ERROR: index_filtered[%d]\n",i);return 1;}
  index_spread_filtered[i]=malloc(MAX_NUMBER_OF_INDEXES_TO_STORE*sizeof(double));if( index_spread_filtered[i]==NULL ){fprintf(stderr,"Memory allocation ERROR: index_spread_filtered[%d]\n",i);return 1;}
  index_spread[i]=malloc(MAX_NUMBER_OF_INDEXES_TO_STORE*sizeof(double));if( index_spread[i]==NULL ){fprintf(stderr,"Memory allocation ERROR: index_spread[%d]\n",i);return 1;}
  lightcurvefilename[i]=malloc(OUTFILENAME_LENGTH*sizeof(char));if( lightcurvefilename[i]==NULL ){fprintf(stderr,"Memory allocation ERROR: lightcurvefilename[%d]\n",i);return 1;}
  known_variable_lightcurvefilename[i]=malloc(OUTFILENAME_LENGTH*sizeof(char));if( known_variable_lightcurvefilename[i]==NULL ){fprintf(stderr,"Memory allocation ERROR: known_variable_lightcurvefilename[%d]\n",i);return 1;}
 }
 i=0; 
 while( NULL!=fgets(string_to_parse,MAX_STRING_LENGTH_IN_LIGHTCURVESTATS_FILE,lightcurve_statistics_file) ){
  string_to_parse[MAX_STRING_LENGTH_IN_LIGHTCURVESTATS_FILE-1]='\0'; // just in case
  if( 100>strlen(string_to_parse) ){fprintf(stderr,"ERROR parsing vast_lightcurve_statistics.log string: %s\n",string_to_parse);continue;}
  if( 6>sscanf(string_to_parse,"%lf %lf %lf %lf %s %[^\t\n]", &tmpmag, &tmpsigma, &tmpdouble, &tmpdouble, lightcurvefilename[i], substring_to_parse) ){fprintf(stderr,"ERROR parsing vast_lightcurve_statistics.log string: %s\n",string_to_parse);continue;}
  array_of_magnitudes[i]=tmpmag;
  index[i][0]=tmpsigma;
  for(j=1;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
   index[i][j]=get_index_by_column_number( substring_to_parse, j);
   // ?????????????????
   if( 0==isnormal(index[i][j]) )index[i][j]=INVALID_INDEX_VALUE;
  }
  i++;  
 }
 fclose(lightcurve_statistics_file);

 // Compute the expected index values and their scatter for each star
 // WE RELY ON THE INPUT ARRAY TO BE SORTED IN MAGNITUDE
 
 fprintf(stderr,"Computing the expected values of variability indexes and their scatter as a function of magnitude");
 
 #ifdef VAST_ENABLE_OPENMP
  #ifdef _OPENMP
   #pragma omp parallel private(i,j,start_index,stop_index,varindex_counter,k,data_for_stat)
  #endif
 #endif
 {
  // We allocate it here and de-allcoate at the end of the cycle to make OpenMP parallelization work
  data_for_stat=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double));
  if( data_for_stat==NULL ){fprintf(stderr,"Memory allocation ERROR: data_for_stat\n");exit(1);}
  // !!!
 
  #ifdef VAST_ENABLE_OPENMP
   #ifdef _OPENMP
    #pragma omp for
   #endif
  #endif
  for(i=0;i<n_stars_in_lightcurve_statistics_file;i++){

   //if( i % 100 == 0 )fprintf(stderr,"\rComputing variability indexes for star %5d out of %5d    ",i,n_stars_in_lightcurve_statistics_file);
   if( i % 100 == 0 )fprintf(stderr,".");
   // Determine the range of indexes for the bin around the current star
   start_index=stop_index=0;
   for(j=0;j<n_stars_in_lightcurve_statistics_file;j++){
    if( j<i ){
     if( array_of_magnitudes[i]-array_of_magnitudes[j]>MAG_BIN_HALF_WIDTH ){
      start_index=j;
     }
    }else{
     if( array_of_magnitudes[j]-array_of_magnitudes[i]<MAG_BIN_HALF_WIDTH ){
      stop_index=j;
     }
     else{
      // Make sure we have enough stars within the bin, take a wider bin if necessary
      if( stop_index-start_index<MIN_N_STARS_IN_INDEX_BIN )stop_index=j;
     }
    }
   }
   // done binning, now collect the index values within the bin
   
   // For each index
   for(varindex_counter=0;varindex_counter<MAX_NUMBER_OF_INDEXES_TO_STORE;varindex_counter++){
    // collect index values within the bin into an array
    for(k=0,j=start_index;j<stop_index;j++){
     if( i==j )continue; // don't count yourself!
     data_for_stat[k]=index[j][varindex_counter];
     if( INVALID_INDEX_VALUE==data_for_stat[k] )continue;
     if( 0==isnormal(data_for_stat[k]) )continue; // redundant, done this check before when assigning the INVALID_INDEX_VALUE value
     if( 0.0==data_for_stat[k] )continue; // THIS IS NEW, forbid 0 index falues
     k++;
    }
    // estimate the expected index value and expected variance
    if( k>1 ){
     gsl_sort( data_for_stat, 1, k);
     index_expected[i][varindex_counter]=gsl_stats_median_from_sorted_data( data_for_stat, 1, k);
     index_spread[i][varindex_counter]=esimate_sigma_from_MAD_of_sorted_data( data_for_stat, k);
    }
    else{
     // If we don't have enough points to estimate the index
     index_expected[i][varindex_counter]=0.0;
     index_spread[i][varindex_counter]=0.0;
    }
   }
  }
 
  free(data_for_stat); /// !!!
 } // #pragma omp parallel 

 fprintf(stderr,"\n"); // to terminate the \r output

 // Yes, filtering works but please revise it!!!!!
 fprintf(stderr,"Filtering the magnitude-index curves");
 // Filter the expected index values and spread
 #ifdef VAST_ENABLE_OPENMP
  #ifdef _OPENMP
   #pragma omp parallel private(i,k,bright_star_half_mag_away,star_half_mag_away,j,l,data_for_stat,data_for_stat_spread)
  #endif
 #endif
 {
  // We allocate it here and de-allcoate at the end of the cycle to make OpenMP parallelization work
  data_for_stat=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double));
  if( data_for_stat==NULL ){fprintf(stderr,"Memory allocation ERROR: data_for_stat\n");exit(1);}
  data_for_stat_spread=malloc(n_stars_in_lightcurve_statistics_file*sizeof(double)); 
  if( data_for_stat_spread==NULL ){fprintf(stderr,"Memory allocation ERROR: data_for_stat_spread\n");exit(1);}
  // !!!

  #ifdef VAST_ENABLE_OPENMP
   #ifdef _OPENMP
    #pragma omp for
   #endif
  #endif
  for(i=0;i<n_stars_in_lightcurve_statistics_file;i++){
 
   if( i % 100 == 0 )fprintf(stderr,".");
   for(k=0;k<i;k++){
    if( array_of_magnitudes[i]-array_of_magnitudes[k]<MAG_BIN_HALF_WIDTH ){
    //if( array_of_magnitudes[i]-array_of_magnitudes[k]<2*MAG_BIN_HALF_WIDTH ){
     bright_star_half_mag_away=i-k;
     break;
    }
   }
   for(k=i;k<n_stars_in_lightcurve_statistics_file;k++){
    if( array_of_magnitudes[k]-array_of_magnitudes[i]>MAG_BIN_HALF_WIDTH ){
    //if( array_of_magnitudes[k]-array_of_magnitudes[i]>2*MAG_BIN_HALF_WIDTH ){
     star_half_mag_away=k-i;
     break;
    }
   }
   for(j=0;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
   //for(j=0;j<2;j++){
    l=0;
    for(k=MAX(0,i-MAX(N_STARS_IN_INDEX_BIN,bright_star_half_mag_away));k<MIN(n_stars_in_lightcurve_statistics_file,i+MAX(N_STARS_IN_INDEX_BIN,star_half_mag_away));k++){
     data_for_stat[l]=index_expected[k][j];
     data_for_stat_spread[l]=index_spread[k][j];
     l++;
    }
    //gsl_sort( data_for_stat, 1, l);
    //index_filtered[i][j]=gsl_stats_median_from_sorted_data( data_for_stat, 1, l);
    index_filtered[i][j]=gsl_stats_mean( data_for_stat, 1, l);
    index_spread_filtered[i][j]=gsl_stats_mean( data_for_stat_spread, 1, l);
    //gsl_sort( data_for_stat_spread, 1, l);
    //index_spread_filtered[i][j]=gsl_stats_median_from_sorted_data( data_for_stat_spread, 1, l);
   }
  }

  free(data_for_stat_spread); // !!!
  free(data_for_stat); // !!! 
 } // #pragma omp parallel

 fprintf(stderr,"\n");

 // If there is a file vast_list_of_previously_known_variables.txt with a list of conwn vars
 list_of_known_vars_file=fopen("vast_autocandidates.log","r");
 if( NULL!=list_of_known_vars_file ){
  n_known_variables=0;
  full_string_autocandidates[0]='\0';
  while( NULL!=fgets(full_string_autocandidates,MAX_STRING_LENGTH_IN_LIGHTCURVESTATS_FILE,list_of_known_vars_file) ){
  //while(-1<fscanf(list_of_known_vars_file,"%s %[^\t\n]",known_variable_lightcurvefilename[n_known_variables],comments_string_autocandidates)){
   sscanf(full_string_autocandidates,"%s %[^\t\n]",known_variable_lightcurvefilename[n_known_variables],comments_string_autocandidates);
   // Count this star only if it's actually among the detected ones
   for(i=0;i<n_stars_in_lightcurve_statistics_file;i++){
    if( 0==strncmp(lightcurvefilename[i],known_variable_lightcurvefilename[n_known_variables],OUTFILENAME_LENGTH) ){   
     fprintf(stderr,"Known variable %s \n",known_variable_lightcurvefilename[n_known_variables]);
     n_known_variables++;
     break;
    }
   }
   full_string_autocandidates[0]='\0';
  }
  fprintf(stderr,"Marking %d known variable stars in the field.\n",n_known_variables);
  
  l=0; // use this counter as a flag for Cmax
  for( threshold=0.0; threshold<50.0; threshold+=0.1 ){
  
   fprintf(stderr,"Selecting candidates with threshold=%lf sigma in each index\r",threshold);
  
   // reset the counters for all indexes
   for(j=0;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
    number_of_selected_objects[j]=number_of_selected_variables[j]=0;
   }
   // For each star
   for(i=0;i<n_stars_in_lightcurve_statistics_file;i++){
    // compare its indexes with a threshold
    for(j=0;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
     //if( fabs(index[i][j]-index_filtered[i][j])>threshold*index_spread_filtered[i][j] ){
     if( index[i][j]>index_filtered[i][j]+threshold*index_spread_filtered[i][j] ){
      number_of_selected_objects[j]++; // count this one as selected object
      // check if this is one of the known variables
      for( k=0;k<n_known_variables;k++){
       if( 0==strncmp(known_variable_lightcurvefilename[k],lightcurvefilename[i],OUTFILENAME_LENGTH) ){
        number_of_selected_variables[j]++;
        break; // there should be only one match anyway
       } // if( 0==strncmp(known_variable_lightcurvefilename[k],lightcurvefilename[i],OUTFILENAME_LENGTH) ){
      } // for( k=0;k<n_known_variables;k++){
     } // if( fabs(index[i][j])>index_filtered[i][j]+threshold*index_spread_filtered[i][j] ){
    } // for(j=0;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
   } // for(i=0;i<n_stars_in_lightcurve_statistics_file;i++){
  
   // Print the resulting statistic for each index
   for(j=0;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
    C = (double)number_of_selected_variables[j]/(double)n_known_variables;
    P = (double)number_of_selected_variables[j]/(double)number_of_selected_objects[j];
    F = 2.0 * (C * P)/(C + P);
    //double beta=50.0;
    //F = (1.0+beta*beta) * (C * P)/(C + beta*beta*P);
    // This is just to make the output look better
    if( 0!=isnan(C) )C=0.0;
    if( 0!=isnan(P) )P=0.0;
    if( 0!=isnan(F) )F=0.0;
    //
    get_index_name( j, short_index_name);
    fprintf(stdout,"%6.3lf  %s   %lf %lf %lf   %5d  %5d  %5d\n", threshold, short_index_name, C, P, F, number_of_selected_variables[j], number_of_selected_objects[j], n_known_variables );
    //
    if( F>Fmax[j] ){
     Fmax[j]=F;
     C_at_Fmax[j]=C;
     P_at_Fmax[j]=P;
     threshold_at_Fmax[j]=threshold;
     fraction_of_obj_rejected_at_Fmax[j]=1.0-(double)number_of_selected_objects[j]/(double)n_stars_in_lightcurve_statistics_file;
    }
    // If this is the first iteration (the one with the lowest threshold)
    // set the maximum completeness values Cmax[j]
    if( l==0 ){
     Cmax[j]=C;
    }
    // In case at first iteration C was zero
    if( C>Cmax[j] )Cmax[j]=C;
    // Check if what is the purity if we are still at the maximum completeness level
    if( C==Cmax[j] ){
     P_at_Cmax[j]=P;
     threshold_at_Cmax[j]=threshold;
     fraction_of_obj_rejected_at_Cmax[j]=1.0-(double)number_of_selected_objects[j]/(double)n_stars_in_lightcurve_statistics_file;
    }
   }
  
   l++; // so we don't change Cmax[j] after the first iteration
  
  } // for( threshold=0.1; threshold<5.0; threshold+=1.0 ){
  fprintf(stderr,"\n"); // to terminate the \r output
  
 } // if( NULL!=list_of_known_vars_file ){

 // Write detection efficeincy summary 
 vast_detection_efficiency_log=fopen("vast_detection_efficiency.log","w");
 if( NULL==vast_detection_efficiency_log ){fprintf(stderr,"ERROR: opening file vast_detection_efficiency.log for writing!\n");exit(1);}
 fprintf(vast_detection_efficiency_log,"#   Index    threshold_at_Fmax threshold_at_Cmax C_at_Fmax P_at_Fmax  Fmax     Cmax   P_at_Cmax fraction_of_obj_rejected_at_Cmax fraction_of_obj_rejected_at_Fmax #\n");
 for(j=0;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
  get_index_name( j, short_index_name);
  //fprintf(stderr,"Considering index %02d %s \n", j, short_index_name);
  fprintf(vast_detection_efficiency_log,"  %s    %7.4lf          %7.4lf          %.5lf   %.5lf  %.5lf  %.5lf  %.5lf              %.5lf                       %.5lf\n",short_index_name,threshold_at_Fmax[j],threshold_at_Cmax[j],C_at_Fmax[j],P_at_Fmax[j],Fmax[j],Cmax[j],P_at_Cmax[j],fraction_of_obj_rejected_at_Cmax[j],fraction_of_obj_rejected_at_Fmax[j]);
 }
 fclose(vast_detection_efficiency_log);



 // Print results
 lightcurve_statistics_expected_file=fopen("vast_lightcurve_statistics_expected.log","w");
 lightcurve_statistics_spread_file=fopen("vast_lightcurve_statistics_spread.log","w");
 lightcurve_statistics_expected_plus_spread_file=fopen("vast_lightcurve_statistics_expected_plus_spread.log","w");
 lightcurve_statistics_expected_plus_spread_file_Cmax=fopen("vast_lightcurve_statistics_expected_plus_spread_Cmax.log","w");
 lightcurve_statistics_expected_minus_spread_file=fopen("vast_lightcurve_statistics_expected_minus_spread.log","w");
 lightcurve_statistics_normalized_file=fopen("vast_lightcurve_statistics_normalized.log","w");
 for(i=0;i<n_stars_in_lightcurve_statistics_file;i++){
  fprintf(lightcurve_statistics_expected_file,"%lf %lf 0 0 0 ",array_of_magnitudes[i],index_filtered[i][0]);
  fprintf(lightcurve_statistics_spread_file,"%lf %lf 0 0 0 ",array_of_magnitudes[i],index_spread_filtered[i][0]);
  // Why don't have the same thing within the for loop and start from j=zero ??????????????
  // Ah yes, that is because we have to put 0's in colums that are not indexes...
  //fprintf(lightcurve_statistics_expected_plus_spread_file,"%lf %lf 0 0 0 ",array_of_magnitudes[i],index_filtered[i][0]+index_spread_filtered[i][0]);
  //fprintf(lightcurve_statistics_expected_minus_spread_file,"%lf %lf 0 0 0 ",array_of_magnitudes[i],index_filtered[i][0]-index_spread_filtered[i][0]);
  fprintf(lightcurve_statistics_expected_plus_spread_file,"%lf %lf 0 0 0 ",array_of_magnitudes[i],index_filtered[i][0]+threshold_at_Fmax[0]*index_spread_filtered[i][0]);
  fprintf(lightcurve_statistics_expected_plus_spread_file_Cmax,"%lf %lf 0 0 0 ",array_of_magnitudes[i],index_filtered[i][0]+threshold_at_Cmax[0]*index_spread_filtered[i][0]);
  fprintf(lightcurve_statistics_expected_minus_spread_file,"%lf %lf 0 0 0 ",array_of_magnitudes[i],index_filtered[i][0]-threshold_at_Fmax[0]*index_spread_filtered[i][0]);  
  fprintf(lightcurve_statistics_normalized_file,"%+10.6lf %+12.6lf 0 0 %s ",array_of_magnitudes[i], (index[i][0]-index_filtered[i][0])/index_spread_filtered[i][0], lightcurvefilename[i]);  
  for(j=1;j<MAX_NUMBER_OF_INDEXES_TO_STORE;j++){
   for(output_file_counter=0;output_file_counter<6;output_file_counter++){
    if( output_file_counter==0 ){
     output_filedescriptor=lightcurve_statistics_expected_file;
     tmp_d_value_to_print=index_filtered[i][j];
    }
    if( output_file_counter==1 ){
     output_filedescriptor=lightcurve_statistics_spread_file;
     tmp_d_value_to_print=index_spread_filtered[i][j];
    }
    if( output_file_counter==2 ){
     output_filedescriptor=lightcurve_statistics_expected_plus_spread_file;
     tmp_d_value_to_print=index_filtered[i][j]+threshold_at_Fmax[j]*index_spread_filtered[i][j];
    }
    if( output_file_counter==3 ){
     output_filedescriptor=lightcurve_statistics_expected_plus_spread_file_Cmax;
     tmp_d_value_to_print=index_filtered[i][j]+threshold_at_Cmax[j]*index_spread_filtered[i][j];
    }
    if( output_file_counter==4 ){
     output_filedescriptor=lightcurve_statistics_expected_minus_spread_file;
     tmp_d_value_to_print=index_filtered[i][j]-threshold_at_Fmax[j]*index_spread_filtered[i][j];
    }
    if( output_file_counter==5 ){
     output_filedescriptor=lightcurve_statistics_normalized_file;
     tmp_d_value_to_print=(index[i][j]-index_filtered[i][j])/index_spread_filtered[i][j];
    }

    // Print the result to the file
    if( tmp_d_value_to_print>=100000.0 || tmp_d_value_to_print<=0.0001 ){
     fprintf(output_filedescriptor,"%+12.6lg ",tmp_d_value_to_print);
    }
    else{
     fprintf(output_filedescriptor,"%+12.6lf ",tmp_d_value_to_print);
    }

   }
   /*
   fprintf(lightcurve_statistics_expected_file,"%lf ",index_filtered[i][j]);
   fprintf(lightcurve_statistics_spread_file,"%lf ",index_spread_filtered[i][j]);
   //fprintf(lightcurve_statistics_expected_plus_spread_file,"%lf ",index_filtered[i][j]+index_spread_filtered[i][j]);
   fprintf(lightcurve_statistics_expected_plus_spread_file,"%lf ",index_filtered[i][j]+threshold_at_Fmax[j]*index_spread_filtered[i][j]);
   fprintf(lightcurve_statistics_expected_plus_spread_file_Cmax,"%lf ",index_filtered[i][j]+threshold_at_Cmax[j]*index_spread_filtered[i][j]);
   //fprintf(lightcurve_statistics_expected_minus_spread_file,"%lf ",index_filtered[i][j]-index_spread_filtered[i][j]);
   fprintf(lightcurve_statistics_expected_minus_spread_file,"%lf ",index_filtered[i][j]-threshold_at_Fmax[j]*index_spread_filtered[i][j]);
   //// Special trouble to make the output look good
   tmp_d_value_to_print=(index[i][j]-index_filtered[i][j])/index_spread_filtered[i][j];
   if( tmp_d_value_to_print>=100000.0 || tmp_d_value_to_print<=0.000001 )
    fprintf(lightcurve_statistics_normalized_file,"%+12.6lg ",tmp_d_value_to_print);
   else
    fprintf(lightcurve_statistics_normalized_file,"%+12.6lf ",tmp_d_value_to_print);
   ////
   */
  }
  fprintf(lightcurve_statistics_expected_file,"\n");
  fprintf(lightcurve_statistics_spread_file,"\n");
  fprintf(lightcurve_statistics_expected_plus_spread_file,"\n");
  fprintf(lightcurve_statistics_expected_plus_spread_file_Cmax,"\n");
  fprintf(lightcurve_statistics_expected_minus_spread_file,"\n");
  fprintf(lightcurve_statistics_normalized_file,"\n");
  //fprintf(stdout,"%lf %lf %lf %lf %lf\n",array_of_magnitudes[i],index_expected[i][25],index_spread[i][25],index_filtered[i][25],index_spread_filtered[i][25]);
  //fprintf(stdout,"%lf %lf %lf %lf\n",array_of_magnitudes[i],index_expected[i][0],index_spread[i][0],index_filtered[i][0]);
 }
 fclose(lightcurve_statistics_normalized_file);
 fclose(lightcurve_statistics_expected_minus_spread_file);
 fclose(lightcurve_statistics_expected_plus_spread_file);
 fclose(lightcurve_statistics_expected_plus_spread_file_Cmax);
 fclose(lightcurve_statistics_spread_file);
 fclose(lightcurve_statistics_expected_file);

 
 // Clean up 
 for(i=0;i<n_stars_in_lightcurve_statistics_file;i++){
  free(index[i]);
  free(index_expected[i]);
  free(index_filtered[i]);
  free(index_spread_filtered[i]);
  free(index_spread[i]);
  free(lightcurvefilename[i]);
  free(known_variable_lightcurvefilename[i]);
 }
 free(index);
 free(index_expected);
 free(index_spread_filtered);
 free(index_spread);
 free(index_for_stat);
 free(array_of_magnitudes);
// free(data_for_stat_spread);
// free(data_for_stat);
 free(sort_index);
 free(lightcurvefilename);
 free(known_variable_lightcurvefilename);

 return 0;
}
