#!/usr/bin/env bash
#
# This small script will copy all content of the directory REGION_NAME (previously created bu util/save.sh script)
# to the current directory. Useful to load previously saved work to the VsST working directory.
#
##########################################################################
# function definitions
function check_if_two_files_are_the_same {
 FILE1="$1"
 FILE2="$2"
 which md5sum &>/dev/null
 if [ $? -ne 0 ];then
  echo "Cannot find the command: md5sum"
  echo "Assuming files $FILE1 and $FILE2 are different"
  return 0
 fi
 if [ ! -f "$FILE1" ];then
  return 0
 fi
 if [ ! -f "$FILE2" ];then
  return 0
 fi
 MD5SUM_OF_FILE1=`md5sum $FILE1 | awk '{print $1}'`
 MD5SUM_OF_FILE2=`md5sum $FILE2 | awk '{print $1}'`
 if [ "$MD5SUM_OF_FILE1" != "$MD5SUM_OF_FILE2" ];then
  echo "Files $FILE1 and $FILE2 are not identical"
  return 0
 fi
 return 1
}
##########################################################################

if [ ! -z $1 ];then
 REGION_NAME="$1"
else
 echo "Name of a region to load?"
 read REGION_NAME
fi

# Test if the data directory exist at all
if [ ! -d $REGION_NAME ];then
 echo "Error: cannot find directory $REGION_NAME"
 if [ -f $REGION_NAME ];then
  echo "Error: $REGION_NAME is a file, not a directory."
 fi
 exit 1
fi   

# Remove old data from the current directory in order not to mix up two different data sets
util/clean_data.sh all 

echo "Loading data from $REGION_NAME ... "
# Copy data
for i in "$REGION_NAME"/* ;do
 # do not copy edited files
 echo `basename $i` | grep "_edit.dat" &>/dev/null
 if [ $? -eq 0 ];then
  echo "skipping the edited lightcurve file $i "
  continue
 fi
 # do not copy the source code
 if [ "$i" = "$REGION_NAME/vast_src_$REGION_NAME" ];then
  continue
 fi
 ### interactive copy for some files
 if [ "$i" = "$REGION_NAME/default.sex.PHOTO" ];then
  check_if_two_files_are_the_same $i default.sex.PHOTO
  if [ $? -ne 1 ];then
   cp -i $i .
  fi
 elif [ "$i" = "$REGION_NAME/default.sex" ];then
  check_if_two_files_are_the_same $i default.sex
  if [ $? -ne 1 ];then
   cp -i $i .
  fi
 elif [ "$i" = "$REGION_NAME/default.psfex" ];then
  check_if_two_files_are_the_same $i default.psfex
  if [ $? -ne 1 ];then
   cp -i $i .
  fi
 else
  cp -r $i .
 fi
done && echo Done
